@@ -2,10 +2,13 @@
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 import errno
+import math
 import mimetypes
 import os
+import platform
 import shutil
 import stat
+import subprocess
 import sys
 import warnings
 from datetime import datetime
@@ -16,7 +19,7 @@ from anyio.to_thread import run_sync
 from jupyter_core.paths import exists, is_file_hidden, is_hidden
 from send2trash import send2trash
 from tornado import web
-from traitlets import Bool, TraitError, Unicode, default, validate
+from traitlets import Bool, Int, TraitError, Unicode, default, validate
 
 from jupyter_server import _tz as tz
 from jupyter_server.base.handlers import AuthenticatedFileHandler
@@ -25,7 +28,7 @@ from jupyter_server.utils import to_api_path
 
 from .filecheckpoints import AsyncFileCheckpoints, FileCheckpoints
 from .fileio import AsyncFileManagerMixin, FileManagerMixin
-from .manager import AsyncContentsManager, ContentsManager
+from .manager import AsyncContentsManager, ContentsManager, copy_pat
 
 try:
     from os.path import samefile
@@ -41,6 +44,8 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
     root_dir = Unicode(config=True)
 
+    max_copy_folder_size_mb = Int(500, config=True, help="The max folder size that can be copied")
+
     @default("root_dir")
     def _default_root_dir(self):
         try:
@@ -221,7 +226,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
         four_o_four = "file or directory does not exist: %r" % path
 
-        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        if not self.allow_hidden and is_hidden(os_path, self.root_dir):
             self.log.info("Refusing to serve hidden file or directory %r, via 404 Error", os_path)
             raise web.HTTPError(404, four_o_four)
 
@@ -273,7 +278,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
         if not os.path.isdir(os_path):
             raise web.HTTPError(404, four_o_four)
-        elif is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        elif not self.allow_hidden and is_hidden(os_path, self.root_dir):
             self.log.info("Refusing to serve hidden directory %r, via 404 Error", os_path)
             raise web.HTTPError(404, four_o_four)
 
@@ -287,7 +292,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
                 try:
                     os_path = os.path.join(os_dir, name)
                 except UnicodeDecodeError as e:
-                    self.log.warning("failed to decode filename '%s': %s", name, e)
+                    self.log.warning("failed to decode filename '%s': %r", name, e)
                     continue
 
                 try:
@@ -297,7 +302,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
                     if e.errno == errno.ENOENT:
                         self.log.warning("%s doesn't exist", os_path)
                     elif e.errno != errno.EACCES:  # Don't provide clues about protected files
-                        self.log.warning("Error stat-ing %s: %s", os_path, e)
+                        self.log.warning("Error stat-ing %s: %r", os_path, e)
                     continue
 
                 if (
@@ -309,9 +314,10 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
                     continue
 
                 try:
-                    if self.should_list(name):
-                        if self.allow_hidden or not is_file_hidden(os_path, stat_res=st):
-                            contents.append(self.get(path=f"{path}/{name}", content=False))
+                    if self.should_list(name) and (
+                        self.allow_hidden or not is_file_hidden(os_path, stat_res=st)
+                    ):
+                        contents.append(self.get(path=f"{path}/{name}", content=False))
                 except OSError as e:
                     # ELOOP: recursive symlink, also don't show failure due to permissions
                     if e.errno not in [errno.ELOOP, errno.EACCES]:
@@ -408,7 +414,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         if not self.exists(path):
             raise web.HTTPError(404, four_o_four)
 
-        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        if not self.allow_hidden and is_hidden(os_path, self.root_dir):
             self.log.info("Refusing to serve hidden file or directory %r, via 404 Error", os_path)
             raise web.HTTPError(404, four_o_four)
 
@@ -431,7 +437,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
     def _save_directory(self, os_path, model, path=""):
         """create a directory"""
-        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        if not self.allow_hidden and is_hidden(os_path, self.root_dir):
             raise web.HTTPError(400, "Cannot create directory %r" % os_path)
         if not os.path.exists(os_path):
             with self.perm_to_403():
@@ -454,7 +460,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
         os_path = self._get_os_path(path)
 
-        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        if not self.allow_hidden and is_hidden(os_path, self.root_dir):
             raise web.HTTPError(400, f"Cannot create file or directory {os_path!r}")
 
         self.log.debug("Saving %s", os_path)
@@ -500,7 +506,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         os_path = self._get_os_path(path)
         rm = os.unlink
 
-        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        if not self.allow_hidden and is_hidden(os_path, self.root_dir):
             raise web.HTTPError(400, f"Cannot delete file or directory {os_path!r}")
 
         four_o_four = "file or directory does not exist: %r" % path
@@ -570,9 +576,9 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         new_os_path = self._get_os_path(new_path)
         old_os_path = self._get_os_path(old_path)
 
-        if (
+        if not self.allow_hidden and (
             is_hidden(old_os_path, self.root_dir) or is_hidden(new_os_path, self.root_dir)
-        ) and not self.allow_hidden:
+        ):
             raise web.HTTPError(400, f"Cannot rename file or directory {old_os_path!r}")
 
         # Should we proceed with the move?
@@ -596,12 +602,126 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         """Return the initial API path of  a kernel associated with a given notebook"""
         if self.dir_exists(path):
             return path
-        if "/" in path:
-            parent_dir = path.rsplit("/", 1)[0]
-        else:
-            parent_dir = ""
+        parent_dir = path.rsplit("/", 1)[0] if "/" in path else ""
         return parent_dir
 
+    def copy(self, from_path, to_path=None):
+        """
+        Copy an existing file or directory and return its new model.
+        If to_path not specified, it will be the parent directory of from_path.
+        If copying a file and to_path is a directory, filename/directoryname will increment `from_path-Copy#.ext`.
+        Considering multi-part extensions, the Copy# part will be placed before the first dot for all the extensions except `ipynb`.
+        For easier manual searching in case of notebooks, the Copy# part will be placed before the last dot.
+        from_path must be a full path to a file or directory.
+        """
+        to_path_original = str(to_path)
+        path = from_path.strip("/")
+        if to_path is not None:
+            to_path = to_path.strip("/")
+
+        if "/" in path:
+            from_dir, from_name = path.rsplit("/", 1)
+        else:
+            from_dir = ""
+            from_name = path
+
+        model = self.get(path)
+        # limit the size of folders being copied to prevent a timeout error
+        if model["type"] == "directory":
+            self.check_folder_size(path)
+        else:
+            # let the super class handle copying files
+            return super().copy(from_path=from_path, to_path=to_path)
+
+        is_destination_specified = to_path is not None
+        to_name = copy_pat.sub(".", from_name)
+        if not is_destination_specified:
+            to_path = from_dir
+        if self.dir_exists(to_path):
+            name = copy_pat.sub(".", from_name)
+            to_name = super().increment_filename(name, to_path, insert="-Copy")
+        to_path = f"{to_path}/{to_name}"
+
+        return self._copy_dir(
+            from_path=from_path,
+            to_path_original=to_path_original,
+            to_name=to_name,
+            to_path=to_path,
+        )
+
+    def _copy_dir(self, from_path, to_path_original, to_name, to_path):
+        """
+        handles copying directories
+        returns the model for the copied directory
+        """
+        try:
+            os_from_path = self._get_os_path(from_path.strip("/"))
+            os_to_path = f'{self._get_os_path(to_path_original.strip("/"))}/{to_name}'
+            shutil.copytree(os_from_path, os_to_path)
+            model = self.get(to_path, content=False)
+        except OSError as err:
+            self.log.error(f"OSError in _copy_dir: {err}")
+            raise web.HTTPError(
+                400,
+                f"Can't copy '{from_path}' into Folder '{to_path}'",
+            ) from err
+
+        return model
+
+    def check_folder_size(self, path):
+        """
+        limit the size of folders being copied to be no more than the
+        trait max_copy_folder_size_mb to prevent a timeout error
+        """
+        limit_bytes = self.max_copy_folder_size_mb * 1024 * 1024
+        size = int(self._get_dir_size(self._get_os_path(path)))
+        # convert from KB to Bytes for macOS
+        size = size * 1024 if platform.system() == "Darwin" else size
+
+        if size > limit_bytes:
+            raise web.HTTPError(
+                400,
+                f"""
+                    Can't copy folders larger than {self.max_copy_folder_size_mb}MB,
+                    "{path}" is {self._human_readable_size(size)}
+                """,
+            )
+
+    def _get_dir_size(self, path="."):
+        """
+        calls the command line program du to get the directory size
+        """
+        try:
+            if platform.system() == "Darwin":
+                # retuns the size of the folder in KB
+                result = subprocess.run(["du", "-sk", path], capture_output=True).stdout.split()
+            else:
+                result = subprocess.run(
+                    ["du", "-s", "--block-size=1", path], capture_output=True
+                ).stdout.split()
+
+            self.log.info(f"current status of du command {result}")
+            size = result[0].decode("utf-8")
+        except Exception:
+            self.log.warning(
+                "Not able to get the size of the %s directory. Copying might be slow if the directory is large!",
+                path,
+            )
+            return "0"
+        return size
+
+    def _human_readable_size(self, size):
+        """
+        returns folder size in a human readable format
+        """
+        if size == 0:
+            return "0 Bytes"
+
+        units = ["Bytes", "KB", "MB", "GB", "TB", "PB"]
+        order = int(math.log2(size) / 10) if size else 0
+
+        return "{:.4g} {}".format(size / (1 << (order * 10)), units[order])
+
 
 class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, AsyncContentsManager):
     """An async file contents manager."""
@@ -621,7 +741,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
 
         if not os.path.isdir(os_path):
             raise web.HTTPError(404, four_o_four)
-        elif is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        elif not self.allow_hidden and is_hidden(os_path, self.root_dir):
             self.log.info("Refusing to serve hidden directory %r, via 404 Error", os_path)
             raise web.HTTPError(404, four_o_four)
 
@@ -636,7 +756,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
                 try:
                     os_path = os.path.join(os_dir, name)
                 except UnicodeDecodeError as e:
-                    self.log.warning("failed to decode filename '%s': %s", name, e)
+                    self.log.warning("failed to decode filename '%s': %r", name, e)
                     continue
 
                 try:
@@ -646,7 +766,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
                     if e.errno == errno.ENOENT:
                         self.log.warning("%s doesn't exist", os_path)
                     elif e.errno != errno.EACCES:  # Don't provide clues about protected files
-                        self.log.warning("Error stat-ing %s: %s", os_path, e)
+                        self.log.warning("Error stat-ing %s: %r", os_path, e)
                     continue
 
                 if (
@@ -658,9 +778,10 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
                     continue
 
                 try:
-                    if self.should_list(name):
-                        if self.allow_hidden or not is_file_hidden(os_path, stat_res=st):
-                            contents.append(await self.get(path=f"{path}/{name}", content=False))
+                    if self.should_list(name) and (
+                        self.allow_hidden or not is_file_hidden(os_path, stat_res=st)
+                    ):
+                        contents.append(await self.get(path=f"{path}/{name}", content=False))
                 except OSError as e:
                     # ELOOP: recursive symlink, also don't show failure due to permissions
                     if e.errno not in [errno.ELOOP, errno.EACCES]:
@@ -775,7 +896,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
 
     async def _save_directory(self, os_path, model, path=""):
         """create a directory"""
-        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        if not self.allow_hidden and is_hidden(os_path, self.root_dir):
             raise web.HTTPError(400, "Cannot create hidden directory %r" % os_path)
         if not os.path.exists(os_path):
             with self.perm_to_403():
@@ -840,7 +961,7 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         os_path = self._get_os_path(path)
         rm = os.unlink
 
-        if is_hidden(os_path, self.root_dir) and not self.allow_hidden:
+        if not self.allow_hidden and is_hidden(os_path, self.root_dir):
             raise web.HTTPError(400, f"Cannot delete file or directory {os_path!r}")
 
         if not os.path.exists(os_path):
@@ -914,9 +1035,9 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         new_os_path = self._get_os_path(new_path)
         old_os_path = self._get_os_path(old_path)
 
-        if (
+        if not self.allow_hidden and (
             is_hidden(old_os_path, self.root_dir) or is_hidden(new_os_path, self.root_dir)
-        ) and not self.allow_hidden:
+        ):
             raise web.HTTPError(400, f"Cannot rename file or directory {old_os_path!r}")
 
         # Should we proceed with the move?
@@ -954,8 +1075,124 @@ class AsyncFileContentsManager(FileContentsManager, AsyncFileManagerMixin, Async
         """Return the initial API path of a kernel associated with a given notebook"""
         if await self.dir_exists(path):
             return path
-        if "/" in path:
-            parent_dir = path.rsplit("/", 1)[0]
-        else:
-            parent_dir = ""
+        parent_dir = path.rsplit("/", 1)[0] if "/" in path else ""
         return parent_dir
+
+    async def copy(self, from_path, to_path=None):
+        """
+        Copy an existing file or directory and return its new model.
+        If to_path not specified, it will be the parent directory of from_path.
+        If copying a file and to_path is a directory, filename/directoryname will increment `from_path-Copy#.ext`.
+        Considering multi-part extensions, the Copy# part will be placed before the first dot for all the extensions except `ipynb`.
+        For easier manual searching in case of notebooks, the Copy# part will be placed before the last dot.
+        from_path must be a full path to a file or directory.
+        """
+        to_path_original = str(to_path)
+        path = from_path.strip("/")
+        if to_path is not None:
+            to_path = to_path.strip("/")
+
+        if "/" in path:
+            from_dir, from_name = path.rsplit("/", 1)
+        else:
+            from_dir = ""
+            from_name = path
+
+        model = await self.get(path)
+        # limit the size of folders being copied to prevent a timeout error
+        if model["type"] == "directory":
+            await self.check_folder_size(path)
+        else:
+            # let the super class handle copying files
+            return await AsyncContentsManager.copy(self, from_path=from_path, to_path=to_path)
+
+        is_destination_specified = to_path is not None
+        to_name = copy_pat.sub(".", from_name)
+        if not is_destination_specified:
+            to_path = from_dir
+        if await self.dir_exists(to_path):
+            name = copy_pat.sub(".", from_name)
+            to_name = await super().increment_filename(name, to_path, insert="-Copy")
+        to_path = f"{to_path}/{to_name}"
+
+        return await self._copy_dir(
+            from_path=from_path,
+            to_path_original=to_path_original,
+            to_name=to_name,
+            to_path=to_path,
+        )
+
+    async def _copy_dir(
+        self, from_path: str, to_path_original: str, to_name: str, to_path: str
+    ) -> dict:
+        """
+        handles copying directories
+        returns the model for the copied directory
+        """
+        try:
+            os_from_path = self._get_os_path(from_path.strip("/"))
+            os_to_path = f'{self._get_os_path(to_path_original.strip("/"))}/{to_name}'
+            shutil.copytree(os_from_path, os_to_path)
+            model = await self.get(to_path, content=False)
+        except OSError as err:
+            self.log.error(f"OSError in _copy_dir: {err}")
+            raise web.HTTPError(
+                400,
+                f"Can't copy '{from_path}' into read-only Folder '{to_path}'",
+            ) from err
+
+        return model
+
+    async def check_folder_size(self, path: str) -> None:
+        """
+        limit the size of folders being copied to be no more than the
+        trait max_copy_folder_size_mb to prevent a timeout error
+        """
+        limit_bytes = self.max_copy_folder_size_mb * 1024 * 1024
+
+        size = int(await self._get_dir_size(self._get_os_path(path)))
+        # convert from KB to Bytes for macOS
+        size = size * 1024 if platform.system() == "Darwin" else size
+        if size > limit_bytes:
+            raise web.HTTPError(
+                400,
+                f"""
+                    Can't copy folders larger than {self.max_copy_folder_size_mb}MB,
+                    "{path}" is {await self._human_readable_size(size)}
+                """,
+            )
+
+    async def _get_dir_size(self, path: str = ".") -> str:
+        """
+        calls the command line program du to get the directory size
+        """
+        try:
+            if platform.system() == "Darwin":
+                # retuns the size of the folder in KB
+                result = subprocess.run(["du", "-sk", path], capture_output=True).stdout.split()
+            else:
+                result = subprocess.run(
+                    ["du", "-s", "--block-size=1", path], capture_output=True
+                ).stdout.split()
+
+            self.log.info(f"current status of du command {result}")
+            size = result[0].decode("utf-8")
+        except Exception:
+            self.log.warning(
+                "Not able to get the size of the %s directory. Copying might be slow if the directory is large!",
+                path,
+            )
+            return "0"
+        return size
+
+    async def _human_readable_size(self, size: int) -> str:
+        """
+        returns folder size in a human readable format
+        """
+        if size == 0:
+            return "0 Bytes"
+
+        units = ["Bytes", "KB", "MB", "GB", "TB", "PB"]
+        order = int(math.log2(size) / 10) if size else 0
+
+        return "{:.4g} {}".format(size / (1 << (order * 10)), units[order])
