@@ -410,7 +410,7 @@ class SessionManager(LoggingConfigurable):
                 raise TypeError(msg)
             conditions.append("%s=?" % column)
 
-        query = "SELECT * FROM session WHERE %s" % (" AND ".join(conditions))
+        query = "SELECT * FROM session WHERE %s" % (" AND ".join(conditions))  # noqa
 
         self.cursor.execute(query, list(kwargs.values()))
         try:
@@ -458,7 +458,7 @@ class SessionManager(LoggingConfigurable):
             if column not in self._columns:
                 raise TypeError("No such column: %r" % column)
             sets.append("%s=?" % column)
-        query = "UPDATE session SET %s WHERE session_id=?" % (", ".join(sets))
+        query = "UPDATE session SET %s WHERE session_id=?" % (", ".join(sets))  # noqa
         self.cursor.execute(query, [*list(kwargs.values()), session_id])
 
     async def kernel_culled(self, kernel_id: str) -> bool:
